@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -41,6 +41,7 @@ G_DEFINE_TYPE (GcmCieWidget, gcm_cie_widget, GTK_TYPE_DRAWING_AREA);
 struct GcmCieWidgetPrivate
 {
 	gboolean		 use_grid;
+	gboolean		 use_whitepoint;
 	guint			 chart_width;
 	guint			 chart_height;
 	cairo_t			*cr;
@@ -401,6 +402,7 @@ enum
 {
 	PROP_0,
 	PROP_USE_GRID,
+	PROP_USE_WHITEPOINT,
 	PROP_RED,
 	PROP_GREEN,
 	PROP_BLUE,
@@ -419,6 +421,9 @@ dkp_cie_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec
 	case PROP_USE_GRID:
 		g_value_set_boolean (value, cie->priv->use_grid);
 		break;
+	case PROP_USE_WHITEPOINT:
+		g_value_set_boolean (value, cie->priv->use_whitepoint);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -438,6 +443,9 @@ dkp_cie_set_property (GObject *object, guint prop_id, const GValue *value, GPara
 	case PROP_USE_GRID:
 		cie->priv->use_grid = g_value_get_boolean (value);
 		break;
+	case PROP_USE_WHITEPOINT:
+		cie->priv->use_whitepoint = g_value_get_boolean (value);
+		break;
 	case PROP_RED:
 		xyz = g_value_get_object (value);
 		cie->priv->red_x = gcm_xyz_get_x (xyz);
@@ -490,6 +498,11 @@ gcm_cie_widget_class_init (GcmCieWidgetClass *class)
 					 g_param_spec_boolean ("use-grid", NULL, NULL,
 							       TRUE,
 							       G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_USE_WHITEPOINT,
+					 g_param_spec_boolean ("use-whitepoint", NULL, NULL,
+							       TRUE,
+							       G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_RED,
 					 g_param_spec_object ("red", NULL, NULL,
@@ -524,6 +537,7 @@ gcm_cie_widget_init (GcmCieWidget *cie)
 
 	cie->priv = GCM_CIE_WIDGET_GET_PRIVATE (cie);
 	cie->priv->use_grid = TRUE;
+	cie->priv->use_whitepoint = TRUE;
 	cie->priv->tongue_buffer = g_ptr_array_new_with_free_func (g_free);
 
 	/* default is CIE REC 709 */
@@ -1082,7 +1096,6 @@ gcm_cie_widget_draw_line (GcmCieWidget *cie, cairo_t *cr)
 	/* overdraw lines with nice antialiasing */
 	gcm_cie_widget_draw_tongue_outline (cie, cr);
 	gcm_cie_widget_draw_gamut_outline (cie, cr);
-	gcm_cie_widget_draw_white_point_cross (cie, cr);
 }
 
 /**
@@ -1133,6 +1146,9 @@ gcm_cie_widget_draw_cie (GtkWidget *cie_widget, cairo_t *cr)
 
 	gcm_cie_widget_draw_line (cie, cr);
 
+	if (cie->priv->use_whitepoint)
+		gcm_cie_widget_draw_white_point_cross (cie, cr);
+
 	cairo_restore (cr);
 }
 
