@@ -12,11 +12,11 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more client.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -36,6 +36,7 @@
 #include <math.h>
 
 #include "gcm-client.h"
+#include "gcm-screen.h"
 #include "gcm-utils.h"
 #include "gcm-edid.h"
 #include "gcm-dmi.h"
@@ -46,6 +47,8 @@ static void     gcm_client_finalize	(GObject     *object);
 
 #define GCM_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GCM_TYPE_CLIENT, GcmClientPrivate))
 
+static void gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output);
+
 /**
  * GcmClientPrivate:
  *
@@ -56,7 +59,7 @@ struct _GcmClientPrivate
 	gchar				*display_name;
 	GPtrArray			*array;
 	GUdevClient			*gudev_client;
-	GnomeRRScreen			*rr_screen;
+	GcmScreen			*screen;
 	GcmEdid				*edid;
 	GcmDmi				*dmi;
 };
@@ -185,7 +188,7 @@ out:
  * gcm_client_gudev_add_type:
  **/
 static void
-gcm_client_gudev_add_type (GcmClient *client, GUdevDevice *udev_device, GcmDeviceType type)
+gcm_client_gudev_add_type (GcmClient *client, GUdevDevice *udev_device, GcmDeviceTypeEnum type)
 {
 	gchar *title;
 	GcmDevice *device = NULL;
@@ -287,32 +290,12 @@ gcm_client_gudev_add (GcmClient *client, GUdevDevice *udev_device)
 {
 	const gchar *value;
 
-	/* only matches HP printers, need to expand to all of CUPS printers */
-	value = g_udev_device_get_property (udev_device, "ID_HPLIP");
+	/* matches our udev rules, which we can change without recompiling */
+	value = g_udev_device_get_property (udev_device, "GCM_DEVICE");
 	if (value != NULL) {
-		egg_debug ("found printer device: %s", g_udev_device_get_sysfs_path (udev_device));
-		gcm_client_gudev_add_type (client, udev_device, GCM_DEVICE_TYPE_PRINTER);
-	}
-
-	/* sane is slightly odd in a lowercase property, and "yes" as a value rather than "1" */
-	value = g_udev_device_get_property (udev_device, "libsane_matched");
-	if (value != NULL) {
-		egg_debug ("found scanner device: %s", g_udev_device_get_sysfs_path (udev_device));
-		gcm_client_gudev_add_type (client, udev_device, GCM_DEVICE_TYPE_SCANNER);
-	}
-
-	/* only matches cameras with gphoto drivers */
-	value = g_udev_device_get_property (udev_device, "ID_GPHOTO2");
-	if (value != NULL) {
-		egg_debug ("found gphoto camera device: %s", g_udev_device_get_sysfs_path (udev_device));
-		gcm_client_gudev_add_type (client, udev_device, GCM_DEVICE_TYPE_CAMERA);
-	}
-
-	/* only matches cameras with v4l devices */
-	value = g_udev_device_get_property (udev_device, "ID_V4L_PRODUCT");
-	if (value != NULL) {
-		egg_debug ("found v4l camera device: %s", g_udev_device_get_sysfs_path (udev_device));
-		gcm_client_gudev_add_type (client, udev_device, GCM_DEVICE_TYPE_CAMERA);
+		value = g_udev_device_get_property (udev_device, "GCM_TYPE");
+		egg_debug ("found %s device: %s", value, g_udev_device_get_sysfs_path (udev_device));
+		gcm_client_gudev_add_type (client, udev_device, gcm_device_type_enum_from_string (value));
 	}
 }
 
@@ -373,6 +356,7 @@ gcm_client_get_output_name (GcmClient *client, GnomeRROutput *output)
 	gboolean ret;
 	guint width = 0;
 	guint height = 0;
+	const guint8 *data;
 	GcmClientPrivate *priv = client->priv;
 
 	/* blank */
@@ -383,14 +367,27 @@ gcm_client_get_output_name (GcmClient *client, GnomeRROutput *output)
 	if (!ret)
 		goto out;
 
+	/* parse the EDID to get a crtc-specific name, not an output specific name */
+	data = gnome_rr_output_get_edid_data (output);
+	if (data != NULL) {
+		ret = gcm_edid_parse (priv->edid, data, NULL);
+		if (!ret) {
+			egg_warning ("failed to parse edid");
+			goto out;
+		}
+	} else {
+		/* reset, as not available */
+		gcm_edid_reset (priv->edid);
+	}
+
 	/* this is an internal panel, use the DMI data */
 	output_name = gnome_rr_output_get_name (output);
 	ret = gcm_utils_output_is_lcd_internal (output_name);
 	if (ret) {
 		/* find the machine details */
 		g_object_get (priv->dmi,
-			      "product-name", &name,
-			      "product-version", &vendor,
+			      "name", &name,
+			      "vendor", &vendor,
 			      NULL);
 
 		/* TRANSLATORS: this is the name of the internal panel */
@@ -435,53 +432,62 @@ out:
 }
 
 /**
- * gcm_client_get_output_id:
- *
- * Return value: the output name, free with g_free().
+ * gcm_client_get_id_for_xrandr_device:
  **/
 static gchar *
-gcm_client_get_output_id (GcmClient *client, GnomeRROutput *output)
+gcm_client_get_id_for_xrandr_device (GcmClient *client, GnomeRROutput *output)
 {
 	const gchar *output_name;
 	gchar *name = NULL;
 	gchar *vendor = NULL;
+	gchar *ascii = NULL;
+	gchar *serial = NULL;
+	const guint8 *data;
 	GString *string;
 	gboolean ret;
-	guint width = 0;
-	guint height = 0;
 	GcmClientPrivate *priv = client->priv;
 
 	/* blank */
-	string = g_string_new ("");
+	string = g_string_new ("xrandr");
 
 	/* if nothing connected then fall back to the connector name */
 	ret = gnome_rr_output_is_connected (output);
 	if (!ret)
 		goto out;
 
+	/* parse the EDID to get a crtc-specific name, not an output specific name */
+	data = gnome_rr_output_get_edid_data (output);
+	if (data != NULL) {
+		ret = gcm_edid_parse (priv->edid, data, NULL);
+		if (!ret) {
+			egg_warning ("failed to parse edid");
+			goto out;
+		}
+	} else {
+		/* reset, as not available */
+		gcm_edid_reset (priv->edid);
+	}
+
 	/* find the best option */
 	g_object_get (priv->edid,
-		      "width", &width,
-		      "height", &height,
 		      "monitor-name", &name,
 		      "vendor-name", &vendor,
+		      "ascii-string", &ascii,
+		      "serial-number", &serial,
 		      NULL);
 
-	/* find the best option */
-	if (name == NULL)
-		g_object_get (priv->edid, "ascii-string", &name, NULL);
-	if (name == NULL)
-		g_object_get (priv->edid, "serial-number", &name, NULL);
-
-	/* prepend vendor if available */
-	if (vendor != NULL && name != NULL)
-		g_string_append_printf (string, "%s - %s", vendor, name);
-	else if (name != NULL)
-		g_string_append_printf (string, "%s", name);
-
+	/* append data if available */
+	if (vendor != NULL)
+		g_string_append_printf (string, "_%s", vendor);
+	if (name != NULL)
+		g_string_append_printf (string, "_%s", name);
+	if (ascii != NULL)
+		g_string_append_printf (string, "_%s", ascii);
+	if (serial != NULL)
+		g_string_append_printf (string, "_%s", serial);
 out:
 	/* fallback to the output name */
-	if (string->len == 0) {
+	if (string->len == 6) {
 		output_name = gnome_rr_output_get_name (output);
 		ret = gcm_utils_output_is_lcd_internal (output_name);
 		if (ret)
@@ -489,40 +495,133 @@ out:
 		g_string_append (string, output_name);
 	}
 
-	/* append size if available */
-	if (width != 0 && height != 0) {
-		guint diag;
-
-		/* calculate the dialgonal using Pythagorean theorem */
-		diag = sqrtf ((powf (width,2)) + (powf (height, 2)));
-
-		/* print it in inches */
-		g_string_append_printf (string, " %i", (guint) ((gfloat) diag * 0.393700787f));
-	}
+	/* replace unsafe chars */
+	gcm_utils_alphanum_lcase (string->str);
 
 	g_free (name);
 	g_free (vendor);
+	g_free (ascii);
+	g_free (serial);
 	return g_string_free (string, FALSE);
 }
 
 /**
- * gcm_client_get_id_for_xrandr_device:
+ * gcm_client_get_device_by_window_covered:
  **/
-static gchar *
-gcm_client_get_id_for_xrandr_device (GcmClient *client, GnomeRROutput *output)
+static gfloat
+gcm_client_get_device_by_window_covered (gint x, gint y, gint width, gint height,
+				         gint window_x, gint window_y, gint window_width, gint window_height)
 {
-	gchar *id;
-	gchar *name;
+	gfloat covered = 0.0f;
+	gint overlap_x;
+	gint overlap_y;
 
-	/* get id */
-	name = gcm_client_get_output_id (client, output);
-	id = g_strdup_printf ("xrandr_%s", name);
+	/* to the right of the window */
+	if (window_x > x + width)
+		goto out;
+	if (window_y > y + height)
+		goto out;
 
-	/* replace unsafe chars */
-	gcm_utils_alphanum_lcase (id);
+	/* to the left of the window */
+	if (window_x + window_width < x)
+		goto out;
+	if (window_y + window_height < y)
+		goto out;
 
-	g_free (name);
-	return id;
+	/* get the overlaps */
+	overlap_x = MIN((window_x + window_width - x), width) - MAX(window_x - x, 0);
+	overlap_y = MIN((window_y + window_height - y), height) - MAX(window_y - y, 0);
+
+	/* not in this window */
+	if (overlap_x <= 0)
+		goto out;
+	if (overlap_y <= 0)
+		goto out;
+
+	/* get the coverage */
+	covered = (gfloat) (overlap_x * overlap_y) / (gfloat) (window_width * window_height);
+	egg_debug ("overlap_x=%i,overlap_y=%i,covered=%f", overlap_x, overlap_y, covered);
+out:
+	return covered;
+}
+
+/**
+ * gcm_client_get_device_by_window:
+ **/
+GcmDevice *
+gcm_client_get_device_by_window (GcmClient *client, GdkWindow *window)
+{
+	gfloat covered;
+	gfloat covered_max = 0.0f;
+	gint window_width, window_height;
+	gint window_x, window_y;
+	gint x, y;
+	guint i;
+	guint len = 0;
+	guint width, height;
+	GnomeRRMode *mode;
+	GnomeRROutput *output_best = NULL;
+	GnomeRROutput **outputs;
+	GcmDevice *device = NULL;
+	gchar *id = NULL;
+
+	/* get the window parameters, in root co-ordinates */
+	gdk_window_get_origin (window, &window_x, &window_y);
+	gdk_drawable_get_size (GDK_DRAWABLE(window), &window_width, &window_height);
+
+	/* get list of updates */
+	outputs = gcm_screen_get_outputs (client->priv->screen, NULL);
+	if (outputs == NULL)
+		goto out;
+
+	/* find length */
+	for (i=0; outputs[i] != NULL; i++)
+		len++;
+
+	/* go through each option */
+	for (i=0; i<len; i++) {
+
+		/* not interesting */
+		if (!gnome_rr_output_is_connected (outputs[i]))
+			continue;
+
+		/* get details about the output */
+		gnome_rr_output_get_position (outputs[i], &x, &y);
+		mode = gnome_rr_output_get_current_mode (outputs[i]);
+		width = gnome_rr_mode_get_width (mode);
+		height = gnome_rr_mode_get_height (mode);
+		egg_debug ("%s: %ix%i -> %ix%i (%ix%i -> %ix%i)", gnome_rr_output_get_name (outputs[i]),
+			   x, y, x+width, y+height,
+			   window_x, window_y, window_x+window_width, window_y+window_height);
+
+		/* get the fraction of how much the window is covered */
+		covered = gcm_client_get_device_by_window_covered (x, y, width, height,
+								   window_x, window_y, window_width, window_height);
+
+		/* keep a running total of which one is best */
+		if (covered > 0.01f && covered > covered_max) {
+			output_best = outputs[i];
+
+			/* optimize */
+			if (covered > 0.99) {
+				egg_debug ("all in one window, no need to search other windows");
+				goto out;
+			}
+
+			/* keep looking */
+			covered_max = covered;
+			egg_debug ("personal best of %f for %s", covered, gnome_rr_output_get_name (output_best));
+		}
+	}
+out:
+	/* if we found an output, get the device */
+	if (output_best != NULL) {
+		id = gcm_client_get_id_for_xrandr_device (client, output_best);
+		egg_debug ("id: %s", id);
+		device = gcm_client_get_device_by_id (client, id);
+	}
+	g_free (id);
+	return device;
 }
 
 /**
@@ -606,7 +705,7 @@ gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output)
 	ret = gcm_utils_output_is_lcd_internal (output_name);
 	if (ret && model == NULL) {
 		g_object_get (priv->dmi,
-			      "product-version", &model,
+			      "version", &model,
 			      NULL);
 	}
 
@@ -614,7 +713,7 @@ gcm_client_xrandr_add (GcmClient *client, GnomeRROutput *output)
 	device = gcm_device_new ();
 	title = gcm_client_get_output_name (client, output);
 	g_object_set (device,
-		      "type", GCM_DEVICE_TYPE_DISPLAY,
+		      "type", GCM_DEVICE_TYPE_ENUM_DISPLAY,
 		      "id", id,
 		      "connected", TRUE,
 		      "serial", serial,
@@ -648,23 +747,6 @@ out:
 	g_free (title);
 }
 
-/**
- * gcm_client_randr_event_cb:
- **/
-static void
-gcm_client_randr_event_cb (GnomeRRScreen *screen, GcmClient *client)
-{
-	GnomeRROutput **outputs;
-	guint i;
-
-	egg_debug ("screens may have changed");
-
-	/* replug devices */
-	outputs = gnome_rr_screen_list_outputs (client->priv->rr_screen);
-	for (i=0; outputs[i] != NULL; i++)
-		gcm_client_xrandr_add (client, outputs[i]);
-}
-
 /**
  * gcm_client_add_connected_devices_xrandr:
  **/
@@ -675,7 +757,9 @@ gcm_client_add_connected_devices_xrandr (GcmClient *client, GError **error)
 	guint i;
 	GcmClientPrivate *priv = client->priv;
 
-	outputs = gnome_rr_screen_list_outputs (priv->rr_screen);
+	outputs = gcm_screen_get_outputs (priv->screen, error);
+	if (outputs == NULL)
+		return FALSE;
 	for (i=0; outputs[i] != NULL; i++)
 		gcm_client_xrandr_add (client, outputs[i]);
 	return TRUE;
@@ -689,7 +773,7 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 {
 	gchar *title;
 	gchar *type_text = NULL;
-	GcmDeviceType type;
+	GcmDeviceTypeEnum type;
 	GcmDevice *device = NULL;
 	gboolean ret;
 	GError *error = NULL;
@@ -700,7 +784,7 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 	if (title == NULL)
 		goto out;
 	type_text = g_key_file_get_string (keyfile, id, "type", NULL);
-	type = gcm_device_type_from_text (type_text);
+	type = gcm_device_type_enum_from_string (type_text);
 
 	/* create device */
 	device = gcm_device_new ();
@@ -709,6 +793,7 @@ gcm_client_add_unconnected_device (GcmClient *client, GKeyFile *keyfile, const g
 		      "id", id,
 		      "connected", FALSE,
 		      "title", title,
+		      "saved", TRUE,
 		      NULL);
 
 	/* load the device */
@@ -759,8 +844,7 @@ gcm_client_add_saved (GcmClient *client, GError **error)
 	groups = g_key_file_get_groups (keyfile, NULL);
 	if (groups == NULL) {
 		ret = FALSE;
-		if (error != NULL)
-			*error = g_error_new (1, 0, "failed to get groups");
+		g_set_error_literal (error, 1, 0, "failed to get groups");
 		goto out;
 	}
 
@@ -772,6 +856,9 @@ gcm_client_add_saved (GcmClient *client, GError **error)
 			gcm_client_add_unconnected_device (client, keyfile, groups[i]);
 		} else {
 			egg_debug ("found already added %s", groups[i]);
+			g_object_set (device,
+				      "saved", TRUE,
+				      NULL);
 		}
 	}
 out:
@@ -826,16 +913,14 @@ gcm_client_delete_device (GcmClient *client, GcmDevice *device, GError **error)
 		      "id", &id,
 		      NULL);
 	if (connected) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "device is still connected");
+		g_set_error_literal (error, 1, 0, "device is still connected");
 		goto out;
 	}
 
 	/* try to remove from array */
 	ret = g_ptr_array_remove (client->priv->array, device);
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "not found in device array");
+		g_set_error_literal (error, 1, 0, "not found in device array");
 		goto out;
 	}
 
@@ -866,6 +951,11 @@ gcm_client_delete_device (GcmClient *client, GcmDevice *device, GError **error)
 	if (!ret)
 		goto out;
 
+	/* update status */
+	g_object_set (device,
+		      "saved", FALSE,
+		      NULL);
+
 	/* emit a signal */
 	egg_debug ("emit removed: %s", id);
 	g_signal_emit (client, signals[SIGNAL_REMOVED], 0, device);
@@ -917,6 +1007,23 @@ gcm_client_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	}
 }
 
+/**
+ * gcm_client_randr_event_cb:
+ **/
+static void
+gcm_client_randr_event_cb (GcmScreen *screen, GcmClient *client)
+{
+	GnomeRROutput **outputs;
+	guint i;
+
+	egg_debug ("screens may have changed");
+
+	/* replug devices */
+	outputs = gcm_screen_get_outputs (screen, NULL);
+	for (i=0; outputs[i] != NULL; i++)
+		gcm_client_xrandr_add (client, outputs[i]);
+}
+
 /**
  * gcm_client_class_init:
  **/
@@ -966,7 +1073,6 @@ gcm_client_class_init (GcmClientClass *klass)
 static void
 gcm_client_init (GcmClient *client)
 {
-	GError *error = NULL;
 	const gchar *subsystems[] = {"usb", "video4linux", NULL};
 
 	client->priv = GCM_CLIENT_GET_PRIVATE (client);
@@ -974,18 +1080,14 @@ gcm_client_init (GcmClient *client)
 	client->priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	client->priv->edid = gcm_edid_new ();
 	client->priv->dmi = gcm_dmi_new ();
+	client->priv->screen = gcm_screen_new ();
+	g_signal_connect (client->priv->screen, "outputs-changed",
+			  G_CALLBACK (gcm_client_randr_event_cb), client);
 
 	/* use GUdev to find devices */
 	client->priv->gudev_client = g_udev_client_new (subsystems);
 	g_signal_connect (client->priv->gudev_client, "uevent",
 			  G_CALLBACK (gcm_client_uevent_cb), client);
-
-	/* get screen */
-	client->priv->rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), (GnomeRRScreenChanged) gcm_client_randr_event_cb, client, &error);
-	if (client->priv->rr_screen == NULL) {
-		egg_error ("failed to get rr screen: %s", error->message);
-		g_error_free (error);
-	}
 }
 
 /**
@@ -1002,7 +1104,7 @@ gcm_client_finalize (GObject *object)
 	g_object_unref (priv->gudev_client);
 	g_object_unref (priv->edid);
 	g_object_unref (priv->dmi);
-	gnome_rr_screen_destroy (priv->rr_screen);
+	g_object_unref (priv->screen);
 
 	G_OBJECT_CLASS (gcm_client_parent_class)->finalize (object);
 }
