@@ -12,11 +12,11 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more device.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -52,10 +52,11 @@ static void     gcm_device_finalize	(GObject     *object);
 struct _GcmDevicePrivate
 {
 	gboolean			 connected;
+	gboolean			 saved;
 	gfloat				 gamma;
 	gfloat				 brightness;
 	gfloat				 contrast;
-	GcmDeviceType			 type;
+	GcmDeviceTypeEnum		 type;
 	gchar				*id;
 	gchar				*serial;
 	gchar				*manufacturer;
@@ -72,6 +73,7 @@ enum {
 	PROP_TYPE,
 	PROP_ID,
 	PROP_CONNECTED,
+	PROP_SAVED,
 	PROP_SERIAL,
 	PROP_MODEL,
 	PROP_MANUFACTURER,
@@ -88,35 +90,35 @@ enum {
 G_DEFINE_TYPE (GcmDevice, gcm_device, G_TYPE_OBJECT)
 
 /**
- * gcm_device_type_from_text:
+ * gcm_device_type_enum_from_string:
  **/
-GcmDeviceType
-gcm_device_type_from_text (const gchar *type)
+GcmDeviceTypeEnum
+gcm_device_type_enum_from_string (const gchar *type)
 {
 	if (g_strcmp0 (type, "display") == 0)
-		return GCM_DEVICE_TYPE_DISPLAY;
+		return GCM_DEVICE_TYPE_ENUM_DISPLAY;
 	if (g_strcmp0 (type, "scanner") == 0)
-		return GCM_DEVICE_TYPE_SCANNER;
+		return GCM_DEVICE_TYPE_ENUM_SCANNER;
 	if (g_strcmp0 (type, "printer") == 0)
-		return GCM_DEVICE_TYPE_PRINTER;
+		return GCM_DEVICE_TYPE_ENUM_PRINTER;
 	if (g_strcmp0 (type, "camera") == 0)
-		return GCM_DEVICE_TYPE_CAMERA;
-	return GCM_DEVICE_TYPE_UNKNOWN;
+		return GCM_DEVICE_TYPE_ENUM_CAMERA;
+	return GCM_DEVICE_TYPE_ENUM_UNKNOWN;
 }
 
 /**
- * gcm_device_type_to_text:
+ * gcm_device_type_enum_to_string:
  **/
 const gchar *
-gcm_device_type_to_text (GcmDeviceType type)
+gcm_device_type_enum_to_string (GcmDeviceTypeEnum type)
 {
-	if (type == GCM_DEVICE_TYPE_DISPLAY)
+	if (type == GCM_DEVICE_TYPE_ENUM_DISPLAY)
 		return "display";
-	if (type == GCM_DEVICE_TYPE_SCANNER)
+	if (type == GCM_DEVICE_TYPE_ENUM_SCANNER)
 		return "scanner";
-	if (type == GCM_DEVICE_TYPE_PRINTER)
+	if (type == GCM_DEVICE_TYPE_ENUM_PRINTER)
 		return "printer";
-	if (type == GCM_DEVICE_TYPE_CAMERA)
+	if (type == GCM_DEVICE_TYPE_ENUM_CAMERA)
 		return "camera";
 	return "unknown";
 }
@@ -197,6 +199,20 @@ gcm_device_load (GcmDevice *device, GError **error)
 		goto out;
 	}
 
+	/* has key */
+	ret = g_key_file_has_group (file, device->priv->id);
+	if (!ret) {
+		/* not fatal */
+		egg_warning ("failed to find parameters for %s", device->priv->id);
+		ret = TRUE;
+		goto out;
+	}
+
+	/* we are backed by a keyfile */
+	g_object_set (device,
+		      "saved", TRUE,
+		      NULL);
+
 	/* load data */
 	g_free (device->priv->profile_filename);
 	device->priv->profile_filename = g_key_file_get_string (file, device->priv->id, "profile", NULL);
@@ -271,8 +287,7 @@ gcm_device_save (GcmDevice *device, GError **error)
 		file = g_file_new_for_path (dirname);
 		ret = g_file_make_directory_with_parents (file, NULL, &error_local);
 		if (!ret) {
-			if (error != NULL)
-				*error = g_error_new (1, 0, "failed to create config directory: %s", error_local->message);
+			g_set_error (error, 1, 0, "failed to create config directory: %s", error_local->message);
 			g_error_free (error_local);
 			goto out;
 		}
@@ -283,8 +298,7 @@ gcm_device_save (GcmDevice *device, GError **error)
 	if (!ret) {
 		ret = g_file_set_contents (filename, "#created today", -1, &error_local);
 		if (!ret) {
-			if (error != NULL)
-				*error = g_error_new (1, 0, "failed to create dummy header: %s", error_local->message);
+			g_set_error (error, 1, 0, "failed to create dummy header: %s", error_local->message);
 			g_error_free (error_local);
 			goto out;
 		}
@@ -299,8 +313,7 @@ gcm_device_save (GcmDevice *device, GError **error)
 			/* ignore */
 			g_clear_error (&error_local);
 		} else {
-			if (error != NULL)
-				*error = g_error_new (1, 0, "failed to load existing config: %s", error_local->message);
+			g_set_error (error, 1, 0, "failed to load existing config: %s", error_local->message);
 			g_error_free (error_local);
 			goto out;
 		}
@@ -347,14 +360,13 @@ gcm_device_save (GcmDevice *device, GError **error)
 	/* save other properties we'll need if we add this device offline */
 	if (device->priv->title != NULL)
 		g_key_file_set_string (keyfile, device->priv->id, "title", device->priv->title);
-	g_key_file_set_string (keyfile, device->priv->id, "type", gcm_device_type_to_text (device->priv->type));
+	g_key_file_set_string (keyfile, device->priv->id, "type", gcm_device_type_enum_to_string (device->priv->type));
 
 	/* convert to string */
 	data = g_key_file_to_data (keyfile, NULL, &error_local);
 	if (data == NULL) {
 		ret = FALSE;
-		if (error != NULL)
-			*error = g_error_new (1, 0, "failed to convert config: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to convert config: %s", error_local->message);
 		g_error_free (error_local);
 		goto out;
 	}
@@ -362,11 +374,15 @@ gcm_device_save (GcmDevice *device, GError **error)
 	/* save contents */
 	ret = g_file_set_contents (filename, data, -1, &error_local);
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "failed to save config: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to save config: %s", error_local->message);
 		g_error_free (error_local);
 		goto out;
 	}
+
+	/* update status */
+	g_object_set (device,
+		      "saved", TRUE,
+		      NULL);
 out:
 	g_free (data);
 	g_free (filename);
@@ -397,6 +413,9 @@ gcm_device_get_property (GObject *object, guint prop_id, GValue *value, GParamSp
 	case PROP_CONNECTED:
 		g_value_set_boolean (value, priv->connected);
 		break;
+	case PROP_SAVED:
+		g_value_set_boolean (value, priv->saved);
+		break;
 	case PROP_SERIAL:
 		g_value_set_string (value, priv->serial);
 		break;
@@ -453,6 +472,9 @@ gcm_device_set_property (GObject *object, guint prop_id, const GValue *value, GP
 	case PROP_CONNECTED:
 		priv->connected = g_value_get_boolean (value);
 		break;
+	case PROP_SAVED:
+		priv->saved = g_value_get_boolean (value);
+		break;
 	case PROP_SERIAL:
 		g_free (priv->serial);
 		priv->serial = g_strdup (g_value_get_string (value));
@@ -532,6 +554,14 @@ gcm_device_class_init (GcmDeviceClass *klass)
 				      G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_CONNECTED, pspec);
 
+	/**
+	 * GcmDevice:saved:
+	 */
+	pspec = g_param_spec_boolean ("saved", NULL, NULL,
+				      FALSE,
+				      G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_SAVED, pspec);
+
 	/**
 	 * GcmCalibrate:serial:
 	 */
@@ -624,6 +654,8 @@ gcm_device_init (GcmDevice *device)
 	GError *error = NULL;
 	device->priv = GCM_DEVICE_GET_PRIVATE (device);
 	device->priv->id = NULL;
+	device->priv->saved = FALSE;
+	device->priv->connected = FALSE;
 	device->priv->serial = NULL;
 	device->priv->manufacturer = NULL;
 	device->priv->model = NULL;
@@ -692,7 +724,7 @@ gcm_device_test (EggTest *test)
 	gchar *profile;
 	gchar *data;
 	const gchar *type;
-	GcmDeviceType type_enum;
+	GcmDeviceTypeEnum type_enum;
 
 	if (!egg_test_start (test, "GcmDevice"))
 		return;
@@ -704,17 +736,17 @@ gcm_device_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "convert to recognised enum");
-	type_enum = gcm_device_type_from_text ("scanner");
-	egg_test_assert (test, (type_enum == GCM_DEVICE_TYPE_SCANNER));
+	type_enum = gcm_device_type_enum_from_string ("scanner");
+	egg_test_assert (test, (type_enum == GCM_DEVICE_TYPE_ENUM_SCANNER));
 
 	/************************************************************/
 	egg_test_title (test, "convert to unrecognised enum");
-	type_enum = gcm_device_type_from_text ("xxx");
-	egg_test_assert (test, (type_enum == GCM_DEVICE_TYPE_UNKNOWN));
+	type_enum = gcm_device_type_enum_from_string ("xxx");
+	egg_test_assert (test, (type_enum == GCM_DEVICE_TYPE_ENUM_UNKNOWN));
 
 	/************************************************************/
 	egg_test_title (test, "convert from recognised enum");
-	type = gcm_device_type_to_text (GCM_DEVICE_TYPE_SCANNER);
+	type = gcm_device_type_enum_to_string (GCM_DEVICE_TYPE_ENUM_SCANNER);
 	if (g_strcmp0 (type, "scanner") == 0)
 		egg_test_success (test, NULL);
 	else
@@ -722,7 +754,7 @@ gcm_device_test (EggTest *test)
 
 	/************************************************************/
 	egg_test_title (test, "convert from unrecognised enum");
-	type = gcm_device_type_to_text (GCM_DEVICE_TYPE_UNKNOWN);
+	type = gcm_device_type_enum_to_string (GCM_DEVICE_TYPE_ENUM_UNKNOWN);
 	if (g_strcmp0 (type, "unknown") == 0)
 		egg_test_success (test, NULL);
 	else
@@ -730,7 +762,7 @@ gcm_device_test (EggTest *test)
 
 	/* set some properties */
 	g_object_set (device,
-		      "type", GCM_DEVICE_TYPE_SCANNER,
+		      "type", GCM_DEVICE_TYPE_ENUM_SCANNER,
 		      "id", "sysfs_dummy_device",
 		      "connected", TRUE,
 		      "serial", "0123456789",
