@@ -12,11 +12,11 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more profile.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -32,6 +32,7 @@
 #include "gcm-utils.h"
 #include "gcm-profile.h"
 #include "gcm-xserver.h"
+#include "gcm-screen.h"
 
 /**
  * gcm_inspect_print_data_info:
@@ -89,7 +90,7 @@ gcm_inspect_show_x11_atoms (void)
 	GcmXserver *xserver = NULL;
 	GnomeRROutput **outputs;
 	guint i;
-	GnomeRRScreen *rr_screen = NULL;
+	GcmScreen *screen = NULL;
 	const gchar *output_name;
 	gchar *title;
 	GError *error = NULL;
@@ -109,17 +110,15 @@ gcm_inspect_show_x11_atoms (void)
 		gcm_inspect_print_data_info (_("Root window profile (deprecated):"), data, length);
 	}
 
-	/* get screen */
-	rr_screen = gnome_rr_screen_new (gdk_screen_get_default (), NULL, NULL, &error);
-	if (rr_screen == NULL) {
+	/* coldplug devices */
+	screen = gcm_screen_new ();
+	outputs = gcm_screen_get_outputs (screen, &error);
+	if (outputs == NULL) {
 		ret = FALSE;
-		egg_warning ("failed to get rr screen: %s", error->message);
+		egg_warning ("failed to get outputs: %s", error->message);
 		g_error_free (error);
 		goto out;
 	}
-
-	/* coldplug devices */
-	outputs = gnome_rr_screen_list_outputs (rr_screen);
 	for (i=0; outputs[i] != NULL; i++) {
 
 		/* get output name */
@@ -144,8 +143,8 @@ gcm_inspect_show_x11_atoms (void)
 	}
 out:
 	g_free (data);
-	if (rr_screen != NULL)
-		gnome_rr_screen_destroy (rr_screen);
+	if (screen != NULL)
+		g_object_unref (screen);
 	if (xserver != NULL)
 		g_object_unref (xserver);
 	return ret;
@@ -193,7 +192,8 @@ gcm_inspect_show_profiles_for_device (const gchar *sysfs_path)
 				 custom_g_type_string_string, &profile_data_array,
 				 G_TYPE_INVALID);
 	if (!ret) {
-		egg_warning ("failed: %s", error->message);
+		/* TRANSLATORS: the DBus method failed */
+		g_print ("%s: %s\n", _("The request failed"), error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -233,6 +233,56 @@ out:
 	return ret;
 }
 
+/**
+ * gcm_inspect_show_profile_for_window:
+ **/
+static gboolean
+gcm_inspect_show_profile_for_window (guint xid)
+{
+	gboolean ret;
+	DBusGConnection *connection;
+	DBusGProxy *proxy;
+	GError *error = NULL;
+	gchar *profile = NULL;
+
+	/* get a session bus connection */
+	connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+
+	/* connect to the interface */
+	proxy = dbus_g_proxy_new_for_name (connection,
+					   "org.gnome.ColorManager",
+					   "/org/gnome/ColorManager",
+					   "org.gnome.ColorManager");
+
+	/* execute sync method */
+	ret = dbus_g_proxy_call (proxy, "GetProfileForWindow", &error,
+				 G_TYPE_UINT, xid,
+				 G_TYPE_INVALID,
+				 G_TYPE_STRING, &profile,
+				 G_TYPE_INVALID);
+	if (!ret) {
+		/* TRANSLATORS: the DBus method failed */
+		g_print ("%s: %s\n", _("The request failed"), error->message);
+		g_error_free (error);
+		goto out;
+	}
+
+	/* no data */
+	if (profile == NULL) {
+		/* TRANSLATORS: no profile has been asigned to this window */
+		g_print ("%s\n", _("There are no ICC profiles for this window"));
+		goto out;
+	}
+
+	/* TRANSLATORS: this is a list of profiles suitable for the device */
+	g_print ("%s %i\n", _("Suitable profiles for:"), xid);
+	g_print ("1.\t%s\n\t%s\n", "this is a title", profile);
+out:
+	g_object_unref (proxy);
+	g_free (profile);
+	return ret;
+}
+
 /**
  * gcm_inspect_show_profiles_for_type:
  **/
@@ -275,7 +325,8 @@ gcm_inspect_show_profiles_for_type (const gchar *type)
 				 custom_g_type_string_string, &profile_data_array,
 				 G_TYPE_INVALID);
 	if (!ret) {
-		egg_warning ("failed: %s", error->message);
+		/* TRANSLATORS: the DBus method failed */
+		g_print ("%s: %s\n", _("The request failed"), error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -321,9 +372,18 @@ out:
 static void
 gcm_inspect_get_properties_collect_cb (const char *key, const GValue *value, gpointer user_data)
 {
-	if (g_strcmp0 (key, "OutputIntent") == 0) {
+	if (g_strcmp0 (key, "RenderingIntentDisplay") == 0) {
 		/* TRANSLATORS: this is the rendering intent of the output */
-		g_print ("%s %s\n", _("Rendering intent (output):"), g_value_get_string (value));
+		g_print ("%s %s\n", _("Rendering intent (display):"), g_value_get_string (value));
+	} else if (g_strcmp0 (key, "RenderingIntentSoftproof") == 0) {
+		/* TRANSLATORS: this is the rendering intent of the printer */
+		g_print ("%s %s\n", _("Rendering intent (softproof):"), g_value_get_string (value));
+	} else if (g_strcmp0 (key, "ColorspaceRgb") == 0) {
+		/* TRANSLATORS: this is filename of the default colorspace */
+		g_print ("%s %s\n", _("RGB Colorspace:"), g_value_get_string (value));
+	} else if (g_strcmp0 (key, "ColorspaceCmyk") == 0) {
+		/* TRANSLATORS: this is filename of the default colorspace */
+		g_print ("%s %s\n", _("CMYK Colorspace:"), g_value_get_string (value));
 	} else {
 		egg_warning ("unhandled property '%s'", key);
 	}
@@ -358,7 +418,8 @@ gcm_inspect_get_properties (void)
 				 dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_VALUE), &hash,
 				 G_TYPE_INVALID);
 	if (!ret) {
-		egg_warning ("failed: %s", error->message);
+		/* TRANSLATORS: the DBus method failed */
+		g_print ("%s: %s\n", _("The request failed"), error->message);
 		g_error_free (error);
 		goto out;
 	}
@@ -382,9 +443,10 @@ main (int argc, char **argv)
 {
 	gboolean x11 = FALSE;
 	gboolean dump = FALSE;
+	guint xid = 0;
 	gchar *sysfs_path = NULL;
 	gchar *type = NULL;
-	GcmDeviceType type_enum;
+	GcmDeviceTypeEnum type_enum;
 	guint retval = 0;
 	GOptionContext *context;
 
@@ -392,10 +454,13 @@ main (int argc, char **argv)
 		{ "x11", 'x', 0, G_OPTION_ARG_NONE, &x11,
 			/* TRANSLATORS: command line option */
 			_("Show X11 properties"), NULL },
-		{ "device", '\0', 0, G_OPTION_ARG_FILENAME, &sysfs_path,
+		{ "device", '\0', 0, G_OPTION_ARG_STRING, &sysfs_path,
 			/* TRANSLATORS: command line option */
 			_("Get the profiles for a specific device"), NULL },
-		{ "type", '\0', 0, G_OPTION_ARG_FILENAME, &type,
+		{ "xid", '\0', 0, G_OPTION_ARG_INT, &xid,
+			/* TRANSLATORS: command line option */
+			_("Get the profile for a specific window"), NULL },
+		{ "type", '\0', 0, G_OPTION_ARG_STRING, &type,
 			/* TRANSLATORS: command line option */
 			_("Get the profiles for a specific device type"), NULL },
 		{ "dump", 'd', 0, G_OPTION_ARG_NONE, &dump,
@@ -424,9 +489,11 @@ main (int argc, char **argv)
 		gcm_inspect_show_x11_atoms ();
 	if (sysfs_path != NULL)
 		gcm_inspect_show_profiles_for_device (sysfs_path);
+	if (xid != 0)
+		gcm_inspect_show_profile_for_window (xid);
 	if (type != NULL) {
-		type_enum = gcm_device_type_from_text (type);
-		if (type_enum == GCM_DEVICE_TYPE_UNKNOWN) {
+		type_enum = gcm_device_type_enum_from_string (type);
+		if (type_enum == GCM_DEVICE_TYPE_ENUM_UNKNOWN) {
 			/* TRANSLATORS: this is when the user does --type=mickeymouse */
 			g_print ("%s\n", _("Device type not recognized"));
 		} else {
