@@ -12,11 +12,11 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more profile.
+ * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -31,6 +31,7 @@
 
 #include <glib-object.h>
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 
 #include "egg-debug.h"
 
@@ -50,11 +51,13 @@ static void     gcm_profile_finalize	(GObject     *object);
  **/
 struct _GcmProfilePrivate
 {
-	guint				 profile_type;
-	guint				 colorspace;
+	GcmProfileTypeEnum		 profile_type;
+	GcmColorspaceEnum		 colorspace;
 	guint				 size;
+	gboolean			 has_vcgt;
 	gchar				*description;
 	gchar				*filename;
+	GFileMonitor			*monitor;
 	gchar				*copyright;
 	gchar				*manufacturer;
 	gchar				*model;
@@ -77,6 +80,7 @@ enum {
 	PROP_TYPE,
 	PROP_COLORSPACE,
 	PROP_SIZE,
+	PROP_HAS_VCGT,
 	PROP_WHITE_POINT,
 	PROP_BLACK_POINT,
 	PROP_LUMINANCE_RED,
@@ -102,8 +106,7 @@ gcm_profile_parse_data (GcmProfile *profile, const guint8 *data, gsize length, G
 
 	/* do we have support */
 	if (klass->parse_data == NULL) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "no support");
+		g_set_error_literal (error, 1, 0, "no support");
 		goto out;
 	}
 
@@ -123,22 +126,16 @@ gcm_profile_parse (GcmProfile *profile, const gchar *filename, GError **error)
 	gboolean ret;
 	guint length;
 	GError *error_local = NULL;
-	GcmProfilePrivate *priv = profile->priv;
 
 	g_return_val_if_fail (GCM_IS_PROFILE (profile), FALSE);
 	g_return_val_if_fail (filename != NULL, FALSE);
 
 	egg_debug ("loading '%s'", filename);
 
-	/* save */
-	g_free (priv->filename);
-	priv->filename = g_strdup (filename);
-
 	/* load files */
 	ret = g_file_get_contents (filename, &data, (gsize *) &length, &error_local);
 	if (!ret) {
-		if (error != NULL)
-			*error = g_error_new (1, 0, "failed to load profile: %s", error_local->message);
+		g_set_error (error, 1, 0, "failed to load profile: %s", error_local->message);
 		g_error_free (error_local);
 		goto out;
 	}
@@ -147,11 +144,44 @@ gcm_profile_parse (GcmProfile *profile, const gchar *filename, GError **error)
 	ret = gcm_profile_parse_data (profile, (const guint8*)data, length, error);
 	if (!ret)
 		goto out;
+
+	/* save */
+	g_object_set (profile,
+		      "filename", filename,
+		      NULL);
 out:
 	g_free (data);
 	return ret;
 }
 
+/**
+ * gcm_profile_save:
+ **/
+gboolean
+gcm_profile_save (GcmProfile *profile, const gchar *filename, GError **error)
+{
+	gboolean ret = FALSE;
+	GcmProfilePrivate *priv = profile->priv;
+	GcmProfileClass *klass = GCM_PROFILE_GET_CLASS (profile);
+
+	/* not loaded */
+	if (priv->size == 0) {
+		g_set_error_literal (error, 1, 0, "not loaded");
+		goto out;
+	}
+
+	/* do we have support */
+	if (klass->save == NULL) {
+		g_set_error_literal (error, 1, 0, "no support");
+		goto out;
+	}
+
+	/* proxy */
+	ret = klass->save (profile, filename, error);
+out:
+	return ret;
+}
+
 /**
  * gcm_profile_generate_vcgt:
  *
@@ -195,55 +225,24 @@ out:
 }
 
 /**
- * gcm_profile_type_to_text:
+ * gcm_profile_file_monitor_changed_cb:
  **/
-const gchar *
-gcm_profile_type_to_text (GcmProfileType type)
+static void
+gcm_profile_file_monitor_changed_cb (GFileMonitor *monitor, GFile *file, GFile *other_file, GFileMonitorEvent event_type, GcmProfile *profile)
 {
-	if (type == GCM_PROFILE_TYPE_INPUT_DEVICE)
-		return "input-device";
-	if (type == GCM_PROFILE_TYPE_DISPLAY_DEVICE)
-		return "display-device";
-	if (type == GCM_PROFILE_TYPE_OUTPUT_DEVICE)
-		return "output-device";
-	if (type == GCM_PROFILE_TYPE_DEVICELINK)
-		return "devicelink";
-	if (type == GCM_PROFILE_TYPE_COLORSPACE_CONVERSION)
-		return "colorspace-conversion";
-	if (type == GCM_PROFILE_TYPE_ABSTRACT)
-		return "abstract";
-	if (type == GCM_PROFILE_TYPE_NAMED_COLOR)
-		return "named-color";
-	return "unknown";
-}
+	GcmProfilePrivate *priv = profile->priv;
 
-/**
- * gcm_profile_colorspace_to_text:
- **/
-const gchar *
-gcm_profile_colorspace_to_text (GcmProfileColorspace type)
-{
-	if (type == GCM_PROFILE_COLORSPACE_XYZ)
-		return "xyz";
-	if (type == GCM_PROFILE_COLORSPACE_LAB)
-		return "lab";
-	if (type == GCM_PROFILE_COLORSPACE_LUV)
-		return "luv";
-	if (type == GCM_PROFILE_COLORSPACE_YCBCR)
-		return "ycbcr";
-	if (type == GCM_PROFILE_COLORSPACE_YXY)
-		return "yxy";
-	if (type == GCM_PROFILE_COLORSPACE_RGB)
-		return "rgb";
-	if (type == GCM_PROFILE_COLORSPACE_GRAY)
-		return "gray";
-	if (type == GCM_PROFILE_COLORSPACE_HSV)
-		return "hsv";
-	if (type == GCM_PROFILE_COLORSPACE_CMYK)
-		return "cmyk";
-	if (type == GCM_PROFILE_COLORSPACE_CMY)
-		return "cmy";
-	return "unknown";
+	/* ony care about deleted events */
+	if (event_type != G_FILE_MONITOR_EVENT_DELETED)
+		goto out;
+
+	/* just rescan everything */
+	egg_debug ("%s deleted, clearing filename", priv->filename);
+	g_object_set (profile,
+		      "filename", NULL,
+		      NULL);
+out:
+	return;
 }
 
 /**
@@ -283,6 +282,9 @@ gcm_profile_get_property (GObject *object, guint prop_id, GValue *value, GParamS
 	case PROP_SIZE:
 		g_value_set_uint (value, priv->size);
 		break;
+	case PROP_HAS_VCGT:
+		g_value_set_boolean (value, priv->has_vcgt);
+		break;
 	case PROP_WHITE_POINT:
 		g_value_set_object (value, priv->white_point);
 		break;
@@ -312,6 +314,7 @@ gcm_profile_set_property (GObject *object, guint prop_id, const GValue *value, G
 {
 	GcmProfile *profile = GCM_PROFILE (object);
 	GcmProfilePrivate *priv = profile->priv;
+	GFile *file;
 
 	switch (prop_id) {
 	case PROP_COPYRIGHT:
@@ -342,10 +345,6 @@ gcm_profile_set_property (GObject *object, guint prop_id, const GValue *value, G
 		if (priv->description != NULL)
 			gcm_utils_ensure_printable (priv->description);
 
-		/* some profile_lcms1s have _really_ long titles - Microsoft, I'm looking at you... */
-		if (priv->description != NULL)
-			gcm_utils_ensure_sane_length (priv->description, 80);
-
 		/* there's nothing sensible to display */
 		if (priv->description == NULL || priv->description[0] == '\0') {
 			g_free (priv->description);
@@ -360,6 +359,21 @@ gcm_profile_set_property (GObject *object, guint prop_id, const GValue *value, G
 	case PROP_FILENAME:
 		g_free (priv->filename);
 		priv->filename = g_strdup (g_value_get_string (value));
+
+		/* unref old instance */
+		if (priv->monitor != NULL) {
+			g_object_unref (priv->monitor);
+			priv->monitor = NULL;
+		}
+
+		/* setup watch on new profile */
+		if (priv->filename != NULL) {
+			file = g_file_new_for_path (priv->filename);
+			priv->monitor = g_file_monitor_file (file, G_FILE_MONITOR_NONE, NULL, NULL);
+			if (priv->monitor != NULL)
+				g_signal_connect (priv->monitor, "changed", G_CALLBACK(gcm_profile_file_monitor_changed_cb), profile);
+			g_object_unref (file);
+		}
 		break;
 	case PROP_TYPE:
 		priv->profile_type = g_value_get_uint (value);
@@ -370,6 +384,9 @@ gcm_profile_set_property (GObject *object, guint prop_id, const GValue *value, G
 	case PROP_SIZE:
 		priv->size = g_value_get_uint (value);
 		break;
+	case PROP_HAS_VCGT:
+		priv->has_vcgt = g_value_get_boolean (value);
+		break;
 	case PROP_WHITE_POINT:
 		priv->white_point = g_value_dup_object (value);
 		break;
@@ -475,6 +492,14 @@ gcm_profile_class_init (GcmProfileClass *klass)
 				   G_PARAM_READWRITE);
 	g_object_class_install_property (object_class, PROP_SIZE, pspec);
 
+	/**
+	 * GcmProfile:has-vcgt:
+	 */
+	pspec = g_param_spec_boolean ("has-vcgt", NULL, NULL,
+				      FALSE,
+				      G_PARAM_READWRITE);
+	g_object_class_install_property (object_class, PROP_HAS_VCGT, pspec);
+
 	/**
 	 * GcmProfile:white-point:
 	 */
@@ -525,8 +550,9 @@ static void
 gcm_profile_init (GcmProfile *profile)
 {
 	profile->priv = GCM_PROFILE_GET_PRIVATE (profile);
-	profile->priv->profile_type = GCM_PROFILE_TYPE_UNKNOWN;
-	profile->priv->colorspace = GCM_PROFILE_COLORSPACE_UNKNOWN;
+	profile->priv->monitor = NULL;
+	profile->priv->profile_type = GCM_PROFILE_TYPE_ENUM_UNKNOWN;
+	profile->priv->colorspace = GCM_COLORSPACE_ENUM_UNKNOWN;
 	profile->priv->white_point = gcm_xyz_new ();
 	profile->priv->black_point = gcm_xyz_new ();
 	profile->priv->luminance_red = gcm_xyz_new ();
@@ -554,6 +580,8 @@ gcm_profile_finalize (GObject *object)
 	g_object_unref (priv->luminance_red);
 	g_object_unref (priv->luminance_green);
 	g_object_unref (priv->luminance_blue);
+	if (priv->monitor != NULL)
+		g_object_unref (priv->monitor);
 
 	G_OBJECT_CLASS (gcm_profile_parent_class)->finalize (object);
 }
@@ -599,8 +627,8 @@ typedef struct {
 	const gchar *model;
 	const gchar *datetime;
 	const gchar *description;
-	GcmProfileType type;
-	GcmProfileColorspace colorspace;
+	GcmProfileTypeEnum type;
+	GcmColorspaceEnum colorspace;
 	gfloat luminance;
 } GcmProfileTestData;
 
@@ -749,8 +777,8 @@ gcm_profile_test (EggTest *test)
 	test_data.manufacturer = "IEC http://www.iec.ch";
 	test_data.model = "IEC 61966-2.1 Default RGB colour space - sRGB";
 	test_data.description = "Blueish Test";
-	test_data.type = GCM_PROFILE_TYPE_DISPLAY_DEVICE;
-	test_data.colorspace = GCM_PROFILE_COLORSPACE_RGB;
+	test_data.type = GCM_PROFILE_TYPE_ENUM_DISPLAY_DEVICE;
+	test_data.colorspace = GCM_COLORSPACE_ENUM_RGB;
 	test_data.luminance = 0.648454;
 	test_data.datetime = "February  9 1998, 06:49:00 AM";
 	gcm_profile_test_parse_file (test, "bluish.icc", &test_data);
@@ -760,8 +788,8 @@ gcm_profile_test (EggTest *test)
 	test_data.manufacturer = "IEC http://www.iec.ch";
 	test_data.model = "IEC 61966-2.1 Default RGB colour space - sRGB";
 	test_data.description = "ADOBEGAMMA-Test";
-	test_data.type = GCM_PROFILE_TYPE_DISPLAY_DEVICE;
-	test_data.colorspace = GCM_PROFILE_COLORSPACE_RGB;
+	test_data.type = GCM_PROFILE_TYPE_ENUM_DISPLAY_DEVICE;
+	test_data.colorspace = GCM_COLORSPACE_ENUM_RGB;
 	test_data.luminance = 0.648446;
 	test_data.datetime = "August 16 2005, 09:49:54 PM";
 	gcm_profile_test_parse_file (test, "AdobeGammaTest.icm", &test_data);
